--- cvsweb.cgi	2005-01-20 12:53:08.000000000 +0000
+++ cvsweb-inc_rlog.cgi	2005-01-20 14:28:12.000000000 +0000
@@ -202,6 +202,7 @@
 sub startproc(@);
 sub runproc(@);
 sub checkout_to_temp($$$);
+sub get_rlog_incremental($$@);
 
 ##### Start of Configuration Area ########
 
@@ -2452,29 +2453,13 @@
   # If there are no files, we're done.
   return @unreadable unless @files;
 
-  my @cmd = ($CMD{rlog});
-  # Can't use -r<tag> as '-' is allowed in tagnames,
-  # but misinterpreted by rlog.
-  push(@cmd, '-r') unless defined($tag);
-
-  my $fh = do { local (*FH); };
-  if (!open($fh, '-|')) {                       # Child
-    open(STDERR, '>', devnull()) unless $DEBUG; # Ignore rlog's complaints.
-    openOutputFilter();
-    if ($file_list_len && $file_list_len > 1) {
-      while (scalar(@files) > $file_list_len) {  # Process files in chunks.
-        system(@cmd, splice(@files, 0, $file_list_len)) == 0 or exit -1;
-      }
-    }
-    exec(@cmd, @files) or exit -1;
-  }
-  undef @cmd;
+  my @rlog_lines = get_rlog_incremental (100, $tag, @files);
 
   my $state = 'start';
   my ($date, $branchpoint, $branch, $log, @filetags);
   my ($rev, $revision, $revwanted, $filename, $head, $author, $keywordsubst);
 
-  while (<$fh>) {
+  for (@rlog_lines) {
     if ($state eq "start") {
 
       #Next file. Initialize file variables
@@ -2609,15 +2594,6 @@
     }
   }
 
-  my $linesread = $. || 0;
-  close($fh);
-
-  if ($linesread == 0) {
-    fatal('500 Internal Error',
-          'Failed to spawn GNU rlog on <em>"%s"</em>.<br /><br />Did you set the <b><code>@command_path</code></b> in your configuration file correctly? (Currently: "<code>%s</code>")',
-          htmlquote(join(', ', @files)), join(':', @command_path));
-  }
-
   return @unreadable;
 }
 
@@ -4377,6 +4353,51 @@
   return ($exitcode, $errormsg);
 }
 
+
+sub get_rlog_incremental($$@)
+{
+  my $files_at_once = shift;
+  my $tag = shift;
+  my @all_files = @_;
+  my @rlog_lines;
+
+  my @cmd = ($CMD{rlog});
+  # Can't use -r<tag> as '-' is allowed in tagnames,
+  # but misinterpreted by rlog.
+  push(@cmd, '-r') unless defined($tag);
+
+  while( my @files = splice (@all_files, 0, $files_at_once)) {
+
+    my $fh = do { local (*FH); };
+    if (!open($fh, '-|')) {                       # Child
+      open(STDERR, '>', devnull()) unless $DEBUG; # Ignore rlog's complaints.
+      openOutputFilter();
+      if ($file_list_len && $file_list_len > 1) {
+        while (scalar(@files) > $file_list_len) {  # Process files in chunks.
+          system(@cmd, splice(@files, 0, $file_list_len)) == 0 or exit -1;
+        }
+      }
+      exec(@cmd, @files) or exit -1;
+    }
+
+    push @rlog_lines, <$fh>;
+
+    my $linesread = $. || 0;
+    close($fh);
+
+    if ($linesread == 0) {
+      fatal('500 Internal Error',
+            'Failed to spawn GNU rlog on <em>"%s"</em>.<br /><br />Did you set the <b><code>@command_path</code></b> in your configuration file correctly? (Currently: "<code>%s</code>")',
+            htmlquote(join(', ', @files)), join(':', @command_path));
+    }
+  }
+
+  return @rlog_lines;
+}
+
+
+
+
 #
 # Check out a file to a temporary file.
 #
